from django.db.models import *


class Model(Model):
    last_modified = DateTimeField(auto_now=True)

    class Meta:
        abstract = True

    @permalink
    def get_absolute_url(self):
        raise NotImplementedError("Please implement get_absolute_url() for each model.")


from django.db import models
from djangorestframework.resources import ModelResource as DrfModelResource
from serene.serializers import RelatedSerializer


class ModelResource(DrfModelResource):
    exclude = ()
    include = ('links',)
    related_serializer = RelatedSerializer

    def filter_response(self, obj):
        self._links = {}
        return super(ModelResource, self).filter_response(obj)

    def links(self, instance):
        self._links['self'] = {
            'href': self.url(instance),
            'rel': 'self',
            }
        return self._links

    def url(self, instance):
        return instance.get_absolute_url()

    def serialize_val(self, key, obj):
        serialized_val = super(ModelResource, self).serialize_val(key, obj)
        if isinstance(obj, models.Model):
            self._links[key] = {
                'href': serialized_val['links']['self']['href'],
                'rel': key,
                'title': serialized_val['title'],
            }
            return serialized_val
        else:
            return serialized_val


from djangorestframework.mixins import InstanceMixin, DeleteModelMixin, ListModelMixin
from djangorestframework.views import ModelView
from serene.mixins import CreateModelMixin, ReadModelMixin, UpdateModelMixin, UpdateOrCreateModelMixin, PaginatorMixin


class InstanceModelView(InstanceMixin, ReadModelMixin, UpdateModelMixin, DeleteModelMixin, ModelView):
    pass

class CreatableInstanceModelView(InstanceMixin, ReadModelMixin, UpdateOrCreateModelMixin, DeleteModelMixin, ModelView):
    pass

class PaginatedListOrCreateModelView(ListModelMixin, CreateModelMixin, PaginatorMixin, ModelView):
    pass

class ListOrCreateModelView(ListModelMixin, CreateModelMixin, ModelView):
    pass


from django.utils.http import http_date
from djangorestframework import status
from djangorestframework.mixins import (
    ModelMixin as DrfModelMixin,
    ReadModelMixin as DrfReadModelMixin,
    CreateModelMixin as DrfCreateModelMixin,
    PaginatorMixin as DrfPaginatorMixin,
)
from djangorestframework.response import ErrorResponse, Response
from urlobject import URLObject
import time


class ReadModelMixin(DrfReadModelMixin):

    def get(self, request, *args, **kwargs):
        instance = super(ReadModelMixin, self).get(request, *args, **kwargs)
        return Response(content=instance, headers={'Last-Modified': http_date(time.mktime(instance.last_modified.timetuple()))})


class UpdateModelMixin(DrfModelMixin):
    """
    Behavior to update a `model` instance on PUT requests
    """
    def put(self, request, *args, **kwargs):
        model = self.resource.model
        try:
            self.model_instance = self.get_instance(*args, **kwargs)
            for (key, val) in self.CONTENT.items():
                setattr(self.model_instance, key, val)
        except model.DoesNotExist:
            raise ErrorResponse(status.HTTP_404_NOT_FOUND)
        self.model_instance.save()
        return self.model_instance


class UpdateOrCreateModelMixin(DrfModelMixin):
    """
    Behavior to update or create a `model` instance on PUT requests
    """
    def put(self, request, *args, **kwargs):
        model = self.resource.model
        try:
            self.model_instance = self.get_instance(*args, **kwargs)
            for (key, val) in self.CONTENT.items():
                setattr(self.model_instance, key, val)
        except model.DoesNotExist:
            self.model_instance = model(**self.get_instance_data(model, self.CONTENT, *args, **kwargs))
            self.model_instance.save()
            return Response(status.HTTP_201_CREATED, self.model_instance)
        self.model_instance.save()
        return self.model_instance


class CreateModelMixin(DrfCreateModelMixin):

    def post(self, request, *args, **kwargs):
        response = super(CreateModelMixin, self).post(request, *args, **kwargs)
        response.headers.update({
            'Content-Location': self.resource(self).url(response.raw_content)
        })
        return response


class PaginatorMixin(DrfPaginatorMixin):

    def first(self, page):
        """
        Returns a url to the first page of results
        """
        return self.url_with_page_number(1)

    def last(self, page):
        """
        Returns a url to the last page of results
        """
        return self.url_with_page_number(page.paginator.num_pages)

    def url_with_page_number(self, page_number):
        """
        Constructs a url used for getting the next/previous urls,
        replacing page & limit with updated number
        """
        url = URLObject(self.request.build_absolute_uri())

        if page_number != 1:
            url = url.set_query_param('page', unicode(page_number))
        else:
            url = url.del_query_param('page')

        limit = self.get_limit()
        if limit != self.limit:
            url = url.set_query_param('limit', unicode(limit))

        return url

    def serialize_page_info(self, page):
        """
        This is some useful information that is added to the response
        """
        links = {'self': {'href': self.request.build_absolute_uri(), 'rel': 'self'}}

        next_page = self.next(page)
        if next_page:
            links['next'] = {'href': next_page, 'rel': 'next'}

        previous_page = self.previous(page)
        if previous_page:
            links['previous'] = {'href': previous_page, 'rel': 'previous'}

        first_page = self.first(page)
        if first_page:
            links['first'] = {'href': first_page, 'rel': 'first'}

        last_page = self.last(page)
        if last_page:
            links['last'] = {'href': last_page, 'rel': 'last'}

        return {
            'links': links,
            'page': page.number,
            'pages': page.paginator.num_pages,
            'per_page': self.get_limit(),
            'total': page.paginator.count,
        }



from django.db import models
from djangorestframework.serializer import Serializer


class RelatedSerializer(Serializer):

    def serialize_model(self, instance):
        """
        serialize_model serializes both Model and dict.
        """
        if isinstance(instance, models.Model):
            return {
                'id': instance.id,
                'title': unicode(instance),
                'links': {
                    'self': {
                        'href': instance.get_absolute_url(),
                        'rel': 'self',
                    }
                },
            }
        else:
            return super(RelatedSerializer, self).serialize_model(instance)


import os

from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "django-serene",
    version = "0.0.3",
    author = "The Sirisanthana Team",
    author_email = "vsirisanthana@gmail.com",
    description = ("An enhanced djangorestframework. Serene is a RESTful framework with a touch of sirisanthana"),
    long_description = read('README.txt'),
    license = "GPL-3.0",
    keywords = "RESTful django djangorestframework",
    classifiers = [
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: GNU General Public License (GPL)"
        ],
    packages = ['serene', 'serene.tests'],
    install_requires = ['djangorestframework'],
)




from django.conf import settings
from django.test.client import RequestFactory
from django.utils import simplejson as json
from django.utils.http import parse_http_date
from django.utils.unittest.case import TestCase
from djangorestframework.response import ErrorResponse
from djangorestframework.tests.testcases import SettingsTestCase
from djangorestframework.views import View
from serene.mixins import ReadModelMixin, UpdateModelMixin, UpdateOrCreateModelMixin, CreateModelMixin, PaginatorMixin
from serene.resources import ModelResource
from serene.tests.models import DummyModel

import time

class TestMixinsBase(SettingsTestCase):
    def setUp(self):
        super(TestMixinsBase, self).setUp()
        installed_apps = tuple(settings.INSTALLED_APPS) + ('serene.tests',)
        self.settings_manager.set(INSTALLED_APPS=installed_apps)
        self.req = RequestFactory()

class TestReadMixin(TestMixinsBase):

    def test_read_model_mixin_must_return_last_modified_header(self):
        dummy = DummyModel.objects.create(name='my dum dum')

        class DummyResource(ModelResource):
            model = DummyModel

        request = self.req.get('/dummies')
        mixin = ReadModelMixin()
        mixin.resource = DummyResource

        response = mixin.get(request, dummy.id)
        self.assertEquals(dummy.name, response.cleaned_content.name)
        self.assertTrue(response.headers.has_key('Last-Modified'))
        last_modified = response.headers['Last-Modified']
        # no error should occur, meaning we have the right format
        last_modified_datetime = parse_http_date(last_modified)
        self.assertEqual(last_modified_datetime, time.mktime(dummy.last_modified.timetuple()))

class TestUpdateModelMixin(TestMixinsBase):

    def setUp(self):
        super(TestUpdateModelMixin, self).setUp()
        self.dummy = DummyModel.objects.create(name='dummy1')

        class DummyResource(ModelResource):
            model = DummyModel
        self.mixin = UpdateModelMixin()
        self.mixin.resource = DummyResource

    def test_update_model(self):
        """
        Making sure update still working fine
        """
        dummy = DummyModel.objects.get(id=1)

        update_data = {'name': 'updated_name'}
        request = self.req.put('/dummy/1', data=update_data)
        self.mixin.CONTENT = update_data

        response = self.mixin.put(request, id=dummy.id)

        self.assertEqual(dummy.name, 'dummy1')
        self.assertEqual(response.name, 'updated_name')
        self.assertEqual(response.id, dummy.id)

        #get dummy again now it should have the new name
        dummy = DummyModel.objects.get(id=dummy.id)
        self.assertEqual(dummy.name, response.name)

    def test_update_not_exist_model_return_404(self):
        """
        When make a put request to non-existing object in update mixin model, it should return 404
        """

        with self.assertRaises(DummyModel.DoesNotExist):
            DummyModel.objects.get(id=999)

        update_data = {'name': 'new_dummy'}
        request = self.req.put('/dummy/999', data=update_data)
        self.mixin.CONTENT = update_data

        self.assertRaises(ErrorResponse, self.mixin.put, request, id=999)

class TestUpdateOrCreateModelMixin(TestMixinsBase):

    def setUp(self):
        super(TestUpdateOrCreateModelMixin, self).setUp()
        self.dummy = DummyModel.objects.create(name='dummy1')

        class DummyResource(ModelResource):
            model = DummyModel
        self.mixin = UpdateOrCreateModelMixin()
        self.mixin.resource = DummyResource

    def test_update_model(self):
        """
        Making sure update still working fine
        """
        dummy = DummyModel.objects.get(id=1)

        update_data = {'name': 'updated_name'}
        request = self.req.put('/dummy/1', data=update_data)
        self.mixin.CONTENT = update_data

        response = self.mixin.put(request, id=dummy.id)

        self.assertEqual(dummy.name, 'dummy1')
        self.assertEqual(response.name, 'updated_name')
        self.assertEqual(response.id, dummy.id)

        #get dummy again now it should have the new name
        dummy = DummyModel.objects.get(id=dummy.id)
        self.assertEqual(dummy.name, response.name)

    def test_creation_on_put_not_exist_model(self):
        """
        When make a put request to non-existing object in update or create mixin model, it should create new object
        and return 201
        """

        with self.assertRaises(DummyModel.DoesNotExist):
            DummyModel.objects.get(id=999)

        self.assertEqual(DummyModel.objects.count(), 1)
        update_data = {'name': 'new_dummy'}
        request = self.req.put('/dummy/999', data=update_data)
        self.mixin.CONTENT = update_data

        response = self.mixin.put(request, id=999)

        self.assertEqual(response.status, 201)
        new_dummy = DummyModel.objects.get(id=999)
        self.assertEqual(new_dummy.name, 'new_dummy')
        self.assertEqual(DummyModel.objects.count(), 2)

class TestCreateModelMixin(TestMixinsBase):

    def setUp(self):
        super(TestCreateModelMixin, self).setUp()
        self.dummy = DummyModel.objects.create(name='dummy1')

        class DummyResource(ModelResource):
            model = DummyModel

            def url(self, instance):
                return '/dummy/%s' % instance.id
        self.mixin = CreateModelMixin()
        self.mixin.resource = DummyResource

    def test_post_to_create_must_return_content_location_header(self):

        with self.assertRaises(DummyModel.DoesNotExist):
            DummyModel.objects.get(name='new_dummy')

        self.assertEqual(DummyModel.objects.count(), 1)
        form_data = {'name': 'new_dummy'}
        request = self.req.post('/dummies', data=form_data)
        self.mixin.CONTENT = form_data

        response = self.mixin.post(request)
        self.assertEqual(DummyModel.objects.count(), 2)
        self.assertEqual(response.status, 201)
        self.assertEqual(response.cleaned_content.name, 'new_dummy')
        self.assertTrue(response.headers.has_key('Content-Location'))
        self.assertEqual(response.headers['Content-Location'], '/dummy/2')


class MockPaginatorView(PaginatorMixin, View):
    total = 60

    def get(self, request):
        return range(0, self.total)

class TestPaginatorMixin(TestCase):
    def setUp(self):
        self.req = RequestFactory()

    def _assert_links(self, links, rel, href):
        self.assertEqual(links[rel]['rel'], rel)
        self.assertEqual(links[rel]['href'], href)


    def _assert_page_info(self, content, expected_pages, expected_page, expected_total, expected_per_page):
        self.assertEqual(content['page'], expected_page)
        self.assertEqual(content['pages'], expected_pages)
        self.assertEqual(content['per_page'], expected_per_page)
        self.assertEqual(content['total'], expected_total)

    def test_paginator_mixin_return_nav_info_first_page(self):
        """
        Paginator mixin should return navigation links including
        self, first, last, next, previous when applicable
        also information of page, total pages, total number of objects, how many object per page
        - First page should not return  prev navigation link
        """
        request = self.req.get('/paginator')
        response = MockPaginatorView.as_view(limit=10)(request)
        content = json.loads(response.content)

        self._assert_page_info(content, 6, 1, 60, 10)
        self.assertTrue(content.has_key('links'))
        links = content['links']

        #assert self
        self.assertTrue(links.has_key('self'))
        self._assert_links(links, 'self', 'http://testserver/paginator')

        #assert next
        self.assertTrue(links.has_key('next'))
        self._assert_links(links, 'next', 'http://testserver/paginator?page=2')

        #assert no prev
        self.assertFalse(links.has_key('previous'))

        #assert first
        self.assertTrue(links.has_key('first'))
        self._assert_links(links, 'first','http://testserver/paginator')

        #assert last
        self.assertTrue(links.has_key('last'))
        self._assert_links(links, 'last', 'http://testserver/paginator?page=6')

    def test_paginator_mixin_return_nav_info_last_page(self):
        """
        Paginator mixin should return navigation links including
        self, first, last, next, previous when applicable
        - Last page should not have next navigation link
        """
        request = self.req.get('/paginator?page=3')
        response = MockPaginatorView.as_view(limit=20)(request)
        content = json.loads(response.content)

        self._assert_page_info(content, 3, 3, 60, 20)
        self.assertTrue(content.has_key('links'))
        links = content['links']

        #assert self
        self.assertTrue(links.has_key('self'))
        self._assert_links(links, 'self', 'http://testserver/paginator?page=3')

        #assert no next
        self.assertFalse(links.has_key('next'))

        #assert prev
        self.assertTrue(links.has_key('previous'))
        self._assert_links(links, 'previous', 'http://testserver/paginator?page=2')

        #assert first
        self.assertTrue(links.has_key('first'))
        self._assert_links(links, 'first','http://testserver/paginator')

        #assert last
        self.assertTrue(links.has_key('last'))
        self._assert_links(links, 'last', 'http://testserver/paginator?page=3')

    def test_paginator_mixin_return_nav_info_only_one_page(self):
        """
        Paginator mixin should return navigation links.
        If there's only one page, paginator should not return navigation links for prev and next
        """
        request = self.req.get('/paginator')
        response = MockPaginatorView.as_view(limit=60)(request)
        content = json.loads(response.content)

        self._assert_page_info(content, 1, 1, 60, 60)
        self.assertTrue(content.has_key('links'))
        links = content['links']

        #assert self
        self.assertTrue(links.has_key('self'))
        self._assert_links(links, 'self', 'http://testserver/paginator')

        #assert no next
        self.assertFalse(links.has_key('next'))

        #assert no prev
        self.assertFalse(links.has_key('previous'))

        #assert first
        self.assertTrue(links.has_key('first'))
        self._assert_links(links, 'first','http://testserver/paginator')

        #assert last
        self.assertTrue(links.has_key('last'))
        self._assert_links(links, 'last', 'http://testserver/paginator')
